/*
 * Author: Hiroyuki Chishiro
 * License: 2-Clause BSD
 */
#ifndef __RASPBERRY_PI_3_H__
#define __RASPBERRY_PI_3_H__


#define SYMBOL_NAME_LABEL(name) name##:

#define PAGE_SHIFT 12
#define PAGE_SIZE (0x1 << PAGE_SHIFT)
#define PAGE_ALIGNMENT __attribute__((aligned(PAGE_SIZE)))

#define ENTRY(name)                             \
  .globl name;                                  \
  SYMBOL_NAME_LABEL(name)

#define NR_CPUS 5
#define KERNEL_STACK_SIZE 0x1000

#define AARCH64_DAIF_IRQ (0x1 << 1)

#define USER_LEVEL 0

#define SCTLR_ELx_nTWE (0x1 << 18)
#define SCTLR_ELx_nTWI (0x1 << 16)
#define SCTLR_ELx_CP15BEN (0x1 << 5)
#define SCTLR_ELx_SA0 (0x1 << 4)
#define SCTLR_ELx_SA (0x1 << 3)
#define SCTLR_ELx_C (0x1 << 2)

#define CPACR_EL1_FPEN_NOT_CAUSE (0x3 << 20)

#ifndef __ASSEMBLY__

#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>
#include "addr.h"
#include "gpio.h"
#include "uart.h"
#include "videocore_mailbox.h"

extern unsigned char PAGE_ALIGNMENT kernel_stack[NR_CPUS][KERNEL_STACK_SIZE];

static inline unsigned long get_cpu_id(void)
{
  unsigned long reg;
  asm volatile("mrs %0, mpidr_el1" : "=r"(reg));
  reg = ((reg >> 8) & 0x00000000ff000000) | (reg & 0x0000000000ffffff);
  return reg;
}

void clear_bss(void);

#else

.macro get_cpu_id reg reg2
mrs \reg2, mpidr_el1
lsr \reg, \reg2, #8
and \reg2, \reg2, #0xffffff
and \reg, \reg, #0xff000000
orr \reg, \reg, \reg2
.endm


.macro enable_local_irq
msr daifclr, #AARCH64_DAIF_IRQ
.endm

.macro disable_local_irq
msr daifset, #AARCH64_DAIF_IRQ
.endm


#endif /* !__ASSEMBLY__ */


#endif /* __RASPBERRY_PI_3_H__ */

