/*
 * Author: Hiroyuki Chishiro
 * License: 2-Clause BSD
 */
#ifndef __UART_H__
#define __UART_H__

#define UART0_DATA_REG (UART0_BASE + 0x00)
#define UART0_FLAG_REG (UART0_BASE + 0x18)
#define UART0_CTRL_REG (UART0_BASE + 0x30)
#define UART0_IMSC_REG (UART0_BASE + 0x38)
#define UART0_ICR_REG (UART0_BASE + 0x44)


#define UART_FLAG_REG_TRANSMIT_FIFO_FULL (0x1 << 5)

#define UART_CTRL_REG_UART_ENABLE (0x1 << 0)

#define UART_IMSC_REG_RECEIVE_INTERRUPT_MASK (0x1 << 4)

#define UART_DATA_REG_DATA_MASK 0xff

#define PL011_UART_IRQ 25
#define PL011_UART_HZ (4 * 1000 * 1000)

#define CLOCK_ID_UART 0x2


#define IRQ_PENDINGn_SRC(x) (0x1 << ((x) & 0x1f))

#ifndef __ASSEMBLY__

#define mmio_in8(addr) (*(volatile uint8_t *) (uintptr_t) (addr))
#define mmio_in16(addr) (*(volatile uint16_t *) (uintptr_t) (addr))
#define mmio_in32(addr) (*(volatile uint32_t *) (uintptr_t) (addr))
#define mmio_in64(addr) (*(volatile uint64_t *) (uintptr_t) (addr))

#define mmio_out8(addr, data) (*(volatile uint8_t *) (uintptr_t) (addr) = (uint8_t) (data))
#define mmio_out16(addr, data) (*(volatile uint16_t *) (uintptr_t) (addr) = (uint16_t) (data))
#define mmio_out32(addr, data) (*(volatile uint32_t *) (uintptr_t) (addr) = (uint32_t) (data))
#define mmio_out64(addr, data) (*(volatile uint64_t *) (uintptr_t) (addr) = (uint64_t) (data))

void setup_pl011_uart(void);
void myputs(char *s);
void uart_pol_putc(uint8_t c);
void init_uart(void);

static inline void nop(void)
{
  asm volatile("nop");
}

static inline void delay_cycles(unsigned long cycles)
{
  unsigned int i;

  for (i = 0; i < cycles; i++) {
    nop();
  }
}

#endif /* !__ASSEMBLY__ */


#endif /* __UART_H__ */

